#!/usr/bin/perl

use strict;
use warnings;

my @terms = ();
my %OPT   = ();
foreach my $arg ( @ARGV ) {
  if    ( $arg =~ /^\-+(.+?)\=(.+)/ ) { $OPT{$1} = $2; }
  elsif ( $arg =~ /^\-+(.+)/        ) { $OPT{$1} = -1; }
  else { push ( @terms, $arg ); }
  }

unless (defined($terms[1])) {
  usage();
  exit(1);
  }

exit(synthesize_wpa_configuration_file($terms[0], $terms[1], \%OPT));

sub usage {
  print ("usage: $0 <ESSID> <password> [options]");
  }

sub synthesize_wpa_configuration_file {
  my ( $essid, $passwd, $opt_hr ) = @_;
  adie ("Password '$passwd' must have at least 8 characters") unless (length($passwd) >= 8);
  my $config_file = $opt_hr->{'f'} || '/etc/wpa_supplicant.conf';
  my @nw_a = (
    "scan_ssid=0",
    "proto=WPA RSN",
    "key_mgmt=WPA-PSK",
    "pairwise=CCMP TKIP",
    "group=CCMP TKIP WEP104 WEP40"
    );
  foreach my $x (`/usr/sbin/wpa_passphrase "$essid" "$passwd" 2>\&1`) {
    next unless ($x =~ /^\s+([^\=]+\=.+)/);
    chomp($x);
    push (@nw_a, $x);
    }
  push (@nw_a, "priority=0");
  wr_f($config_file, "ctrl_interface=/var/run/wpa_supplicant\nctrl_interface_group=0\neapol_version=1\nap_scan=1\nfast_reauth=1\nnetwork={\n");
  foreach my $x (@nw_a) {
    ap_f($config_file, "$x\n");
    }
  ap_f($config_file, "}\n");
  chmod (0750, $config_file);
  return 0;
  }

sub adie {
  my ( $msg ) = @_;
  print ("ERROR: $msg\n");
  exit(2);
  }

sub wr_f {
  my ( $fn, $buf ) = @_;
  my ( $fh );
  return("ERROR no filename supplied") unless($fn);
  open  ( $fh, ">$fn" ) or return("ERROR cannot open '$fn'");
  print ( $fh $buf );
  close ( $fh );
  return ( 'OK' );
  }

sub ap_f {
  my ( $fn, $buf ) = @_;
  my ( $fh );
  return("ERROR no filename supplied") unless($fn);
  open  ( $fh, ">>$fn" ) or return("ERROR cannot open '$fn'");
  print ( $fh $buf );
  close ( $fh );
  return ( 'OK' );
  }
